#ifndef IO_STL_CONTNR_H_
#define IO_STL_CONTNR_H_

#include <list>
#include <map>
#include <set>
#include <vector>
#include <ostream>

#include "boost/multi_array.hpp"



// *****************************************************************************
// * STL
// *****************************************************************************

template<class T> std::ostream& operator<<(std::ostream& s, const std::vector<T>& v)
{
	for (typename std::vector<T>::const_iterator it = v.begin(); it != v.end(); ++it)
		s << *it << " ";
	return s;
}



template<class T> std::ostream& operator<<(std::ostream& s, const std::vector<T*>& v)
{
	for (typename std::vector<T*>::const_iterator it = v.begin(); it != v.end(); ++it) {
		s << **it << " ";
	}
	return s;
}



template<class T> std::ostream& operator<<(std::ostream& s, const std::list<T>& l)
{
	for (typename std::list<T>::const_iterator it = l.begin(); it != l.end(); ++it)
		s << *it << " ";
	return s;
}



template<class T> std::ostream& operator<<(std::ostream& s, const std::list<T*>& l)
{
	for (typename std::list<T*>::const_iterator it = l.begin(); it != l.end(); ++it) {
		s << **it << " ";
	}
	return s;
}



template<class T> std::ostream& operator<<(std::ostream& s, const std::set<T>& l)
{
	typename std::set<T>::const_iterator it = l.begin();
	s << *it;
	++it;
	for (; it != l.end(); ++it)
		s << " " << *it;
	return s;
}



template<class T> std::ostream& operator<<(std::ostream& s, const std::set<T*>& l)
{
	for (typename std::set<T*>::const_iterator it = l.begin(); it != l.end(); ++it) {
		s << **it << " ";
	}
	return s;
}



template<class K, class V> std::ostream& operator<<(std::ostream& s, const std::map<K, V>& mp)
{
	for (typename std::map<K, V>::const_iterator it = mp.begin(); it != mp.end(); ++it)
		s << (*it).first << " => " << (*it).second << "   ";
	return s;
}



// *****************************************************************************
// * Boost
// *****************************************************************************

std::ostream& operator<<(std::ostream& s, const boost::multi_array<int, 2>& a);
std::ostream& operator<<(std::ostream& s, const boost::multi_array<double, 2>& a);
std::ostream& operator<<(std::ostream& s, const boost::multi_array<int, 3>& a);
std::ostream& operator<<(std::ostream& s, const boost::multi_array<double, 3>& a);

#endif
