% contains all file and directory names
classdef FilesDirs    
    properties (Constant) 
        local_home_dir = '/home/ingo'
        cluster_home_dir = '/gobics/home/ingo'
        scratch_tmp_dir = '/c1/scratch/ingo/small_subtypes'
        repr_em_freqs_filename = 'EF_repr.txt'
        repr_tr_freqs_filename = 'TF_repr.txt'
    end
    
    methods (Static)
        function dirname = non_scratch_tmp_dir()
            dirname = [FilesDirs.home_dir() '/tmp/small_subtypes'];
        end
        function dirname = calc_etp_dir()
            dirname = [FilesDirs.home_dir() '/workspace/small_subtypes/Release'];
        end
        function dirname = ml_ext_dir()
            dirname = [FilesDirs.home_dir() '/workspace/small_subtypes/ml_ext'];
        end
        function dirname = input_dir()
            dirname = [FilesDirs.home_dir() '/workspace/small_subtypes/input'];
        end
        function dirname = local_script_dir()
            dirname = [FilesDirs.local_home_dir '/workspace/small_subtypes/src/scripts'];
        end
        function dirname = script_dir()
            dirname = [FilesDirs.home_dir() '/workspace/small_subtypes/src/scripts'];
        end
        function dirname = mach_learn_em_probs_dir()
            dirname = [FilesDirs.home_dir() '/workspace/small_subtypes/data'];
        end
        function dirname = jphmm_exe_dir()
            dirname = [FilesDirs.home_dir() '/workspace/jpHMM/src/rewrite_jpHMM'];
        end
        function dirname = jphmm_dir()
            dirname = [FilesDirs.home_dir() '/workspace/jpHMM'];
        end
        function dirname = cluster_output_dir()
            dirname = [FilesDirs.scratch_tmp_dir '/cluster/output'];
        end
        function dirname = cluster_error_dir()
            dirname = [FilesDirs.scratch_tmp_dir '/cluster/error'];
        end
        function dirname = matlab_var_dir()
            dirname = [FilesDirs.local_home_dir '/workspace/small_subtypes/data/matlab'];
        end
        function dirname = id_filename()
            dirname = [FilesDirs.local_home_dir '/workspace/small_subtypes/data/id.txt'];
        end
        function filename = jphmm_em_priors_filename()
            filename = [FilesDirs.jphmm_dir '/src/rewrite_jpHMM/nucleic_Alphas_estimated_HIV.txt'];
        end
        function filename = jphmm_tr_priors_filename()
            filename = [FilesDirs.jphmm_dir '/src/rewrite_jpHMM/transition_priors.txt'];
        end
        function filename = wait_script_filename()
            filename = [FilesDirs.script_dir() '/wait_for_file.sh'];
        end

        function dirname = home_dir()
            global consts;
            if consts.use_cluster
                dirname = FilesDirs.cluster_home_dir;
            else
                dirname = FilesDirs.local_home_dir;
            end
        end
        
        function dirname = tmp_dir()
            global consts;
            if consts.use_scratch
                dirname = FilesDirs.scratch_tmp_dir;
            else
                dirname = FilesDirs.non_scratch_tmp_dir;
            end
        end     
        
        function filename = prep_job_local_filename()
            filename = [FilesDirs.local_script_dir '/prep_job_local.txt'];
        end        
        function filename = prep_job_cluster_filename()
            filename = [FilesDirs.local_script_dir '/prep_job_cluster.txt'];
        end
        
        function filename = ef_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/EF/EF_' stamp '.txt'];
        end
        function filename = tf_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/TF/TF_' stamp '.txt'];
        end
        function filename = ep_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/EP/EP_' stamp '.txt'];
        end
        function filename = tp_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/TP/TP_' stamp '.txt'];
        end
        function filename = cons_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/cons/cons_' stamp '.txt'];
        end
        function filename = jpHMM_res_dirname()
            filename = [FilesDirs.tmp_dir() '/jpHMM_results'];
        end
        function filename = jpHMM_res_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/jpHMM_results/recombination_' stamp '.txt'];
        end
        function filename = jpHMM_res_incl_gaps_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/jpHMM_results/recombination_incl_gaps_' stamp '.txt'];
        end
        function filename = jpHMM_al_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/jpHMM_al/AL_' stamp '.fas'];
        end
        function filename = jpHMM_query_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/jpHMM_query/Query_' stamp '.fas'];
        end
        function filename = calc_etp_al_filename(scenario)
            filename = [FilesDirs.tmp_dir() '/calcETP_al/alignment_' scenario '.fas'];
        end
        function filename = calc_etp_seqs_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/calcETP_seqs/seqs_' stamp '.txt'];
        end
        function filename = mod_sel_para_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/calcETP_para/para_' stamp '.txt'];
        end
        function filename = script_filename(name, stamp)
            filename = [FilesDirs.tmp_dir() '/scripts/script_' name '_' stamp '.sh'];
        end
        function filename = mcmc_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/mcmc/mcmc_' stamp '.txt'];
        end
        function filename = curr_mcmc_filename()
            filename = [FilesDirs.tmp_dir() '/mcmc/mcmc.txt'];
        end
        function filename = mach_learn_vars_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/mach_learn_vars/vars_' stamp '.mat'];
        end
        function filename = ml_weights_filename(stamp)
            filename = [FilesDirs.tmp_dir() '/mach_learn_vars/weights_' stamp '.txt'];
        end
       
        % write string to current and stamped mcmc file
        function write2mcmc_files(stamp, str)
            fid = FilesDirs.sec_fopen(FilesDirs.mcmc_filename(stamp), 'a');
            fprintf(fid, '%s\n' , str);
            fclose(fid);
            
            fid = FilesDirs.sec_fopen(FilesDirs.curr_mcmc_filename, 'a');
            fprintf(fid, '%s\n' , str);
            fclose(fid);
        end
        
        % secure file open
        function fid = sec_fopen(filename, mode)
            fid = fopen(filename, mode);
            if fid == -1 
                error(['Unable to open: ' filename]) 
            end
        end
        
        % ???
        function move_to_scratch()
            copy_cmd = ['cp -r ' FilesDirs.tmp_dir() '/* ' FilesDirs.scratch_dir];
            system(copy_cmd);
            delete_cmd = ['find ' FilesDirs.tmp_dir() ' -type f -exec rm {} \;'];
            system(delete_cmd);
            system(['cp ' FilesDirs.scratch_dir '/calcETP_al/*.* ' FilesDirs.tmp_dir() '/calcETP_al']);
        end
    end
end
