% provides function for calling jpHMM
classdef JPHMM
    methods (Static)
        % create a job which calls jpHMM
        function job = create_job(model_para, stamp, dependencies)
            global consts;           
            % call jpHMM
            switch consts.strategy
                case 'orig'
                    cmd = [FilesDirs.jphmm_exe_dir '/jpHMM -i ' FilesDirs.jpHMM_al_filename(stamp) ' -s ' FilesDirs.jpHMM_query_filename(stamp) ...
                           ' -v HIV -j ' num2str(model_para.jp) ' -x ' stamp ' -o ' FilesDirs.jpHMM_res_dirname() ...
                           ' -a ' FilesDirs.jphmm_em_priors_filename ' -b ' FilesDirs.jphmm_tr_priors_filename ...                
                           ' -e ' FilesDirs.ep_filename(stamp) ' -t ' FilesDirs.tp_filename(stamp) ' -c ' FilesDirs.cons_filename(stamp)];                 
                case 'prob'
                    cmd = [FilesDirs.jphmm_exe_dir '/jpHMM -i ' FilesDirs.jpHMM_al_filename(stamp) ' -s ' FilesDirs.jpHMM_query_filename(stamp) ...
                           ' -v HIV -j ' num2str(model_para.jp) ' -x ' stamp ' -o ' FilesDirs.jpHMM_res_dirname() ...
                           ' -a ' FilesDirs.jphmm_em_priors_filename ' -b ' FilesDirs.jphmm_tr_priors_filename ...
                           ' -e ' FilesDirs.ep_filename(stamp) ' -t ' FilesDirs.tp_filename(stamp) ' -c ' FilesDirs.cons_filename(stamp)];                 
                case 'ml'
                    cmd = [FilesDirs.jphmm_exe_dir '/jpHMM -i ' FilesDirs.jpHMM_al_filename(stamp) ' -s ' FilesDirs.jpHMM_query_filename(stamp) ...
                           ' -v HIV -j ' num2str(model_para.jp) ' -x ' stamp ' -o ' FilesDirs.jpHMM_res_dirname() ...
                           ' -a ' FilesDirs.jphmm_em_priors_filename ' -b ' FilesDirs.jphmm_tr_priors_filename ...
                           ' -e ' FilesDirs.ep_filename(stamp) ' -t ' FilesDirs.tp_filename(stamp) ' -c ' FilesDirs.cons_filename(stamp)];                 
                otherwise
                    error 'Invalid strategy'
            end    
            if consts.use_scratch
%                 cmd = [cmd ' -S'];
            end
            disp(cmd)
            job = Job(consts.use_cluster, true, 'testJPHMM', stamp, cmd, dependencies);
        end
        
        % get subtype segmentation for one test sequence yielded by jpHMM
        function segm = results(stamp, begin_pos, end_pos, test_seq)
            WAITING_TIME = 1;
            MAX_WAITING_TIME = 60;
            
            is_ok = false;
            n = 0;
            while (~is_ok);
                [bps, subs] = JPHMM.parse(FilesDirs.jpHMM_res_incl_gaps_filename(stamp));

                if (isempty(subs))
                    [bps, subs] = JPHMM.parse(FilesDirs.jpHMM_res_filename(stamp));
                end   
                n = n + 1;
                if (n * WAITING_TIME > MAX_WAITING_TIME) || ~isempty(subs)
                    is_ok = true;
                else
                    pause(WAITING_TIME);
                end
            end
            
            if (isempty(subs))
                INVALID_SUB = 'Z';
                disp(['no subs found in jpHMM results file (stamp: ' stamp ')'])               
                bps = [];
                subs = {INVALID_SUB};
            else            
                % remove unassigned parts (5' and 3'-Insertion)
                if (~isempty(findstr(subs{1}, '-')))
                    bps = bps(2:end);
                    subs = subs(2:end);
                end
                if (~isempty(findstr(subs{end}, '-')))
                    bps = bps(1:end-1);
                    subs = subs(1:end-1);
                end

                % convert positions
                for i=1:length(bps)                
                    bps(i) = test_seq.pos_with_gaps(bps(i));
                end
            end
            segm = Segmentation(begin_pos, end_pos, bps, subs, begin_pos);
        end
    end
        
    methods (Static, Access = private)
        % parse results file of jpHMM
        function [bps, subs] = parse(filename)
            % resutls file exists?
            fid = fopen(filename);
            if fid == -1 
                error(['Unable to open: ' filename]) 
            end

            % parse results
            bps = [];
            subs = {};           
            while 1
                tline = fgetl(fid);
                if ~ischar(tline), break, end
                if ~isempty(tline) && (tline(1) ~= '#')  && (tline(1) ~= 'Z')                    
                    c = textscan(tline, '%d %d %s');
                    [s1, s2] = size(c{1});
                    if s1
                       bps = [bps c{1}] ;
                       subs = [subs c{3}];
                    end
                end
            end
            bps = bps(2:end);
            fclose(fid);               
        end        
    end
end