#include "consts.h"
#include "error.h"
#include "io_contnr.h"
#include "partition.h"

#include <string>
#include <vector>
#include <set>
#include <map>

#include <boost/assign/list_inserter.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/assign/std/vector.hpp>
#include <boost/assign/std/set.hpp>
#include <boost/foreach.hpp>

using namespace std;
using namespace boost;
using namespace boost::assign;

#define foreach BOOST_FOREACH
#define comma ,

namespace partition 
{

/// get source partition induced by source vector (input (0, 0, 1, 0, 2, 1) yields (0 -> {0, 1, 3}, 1 -> {2, 5}, 2 -> {4}))
map<unsigned, set<unsigned> > get_source_partition(vector<unsigned> v)
{
	map<unsigned, set<unsigned> > ret;
	for (unsigned i = 0; i < v.size(); ++i)
		ret[v[i]] += i;
	return ret;
}


/// get source vector induced by source partition (input (0 -> {0, 1, 3}, 1 -> {2, 5}, 2 -> {4}) yields (0, 0, 1, 0, 2, 1))
vector<unsigned> get_source_vector(map<unsigned, set<unsigned> > prt)
{
	map<unsigned, unsigned> mp;
	foreach (pair<unsigned comma set<unsigned> > pr, prt)
		foreach (unsigned sub, pr.second)
			mp[sub] = pr.first;

	vector<unsigned> ret(mp.size());
	foreach (pair<unsigned comma unsigned> pr, mp)
		ret[pr.first] = pr.second;
	return ret;
}


/// get all possible combinations of sources 
/// the total number of sources in the combinations has to be num_sources
vector< map<unsigned, set<unsigned> > > get_all_source_partitions(unsigned num_sources, unsigned num_subs, unsigned fixed_sub)
{
	vector< vector<unsigned> > source_combs;
	for (unsigned n = 0; n < pow((double)num_sources, num_subs - 1); ++n) {
		unsigned m = n;
		vector<unsigned> curr_sources(num_subs);
		for (unsigned i = 0; i < num_subs; ++i) {
			if (i == fixed_sub)
				curr_sources[i] = 0;
			else {
				curr_sources[i] = m % num_sources;
				m /= num_sources;
			}
		}

		bool is_ok = true;
		unsigned max_sub = 0;
		for (unsigned i = 0; i < num_subs; ++i) {
			if (i != fixed_sub) {
				if (curr_sources[i] > max_sub + 1)
					is_ok = false;
				max_sub = max(max_sub, curr_sources[i]);
			}
		}
		if (num_sources != max_sub + 1)
			is_ok = false;

		if (is_ok) {
			source_combs += curr_sources;
		}
	}

	vector< map<unsigned, set<unsigned> > > ret;
	foreach (vector<unsigned> v, source_combs)
		ret += get_source_partition(v);
	return ret;
}


/// get combinations of sources derived from a given one
vector< map<unsigned, set<unsigned> > > get_derived_source_partitions(map<unsigned, set<unsigned> > source_partition)
{
	vector< map<unsigned, set<unsigned> > > ret;
	foreach (pair<unsigned comma set<unsigned> > pr, source_partition) {
		set<unsigned> curr_subs = pr.second;
		map<unsigned, set<unsigned> > curr_partition = source_partition;
		for (unsigned n = 1; n < pow(2, curr_subs.size() - 1); ++n) {
			set<unsigned> subs_staying = list_of(*curr_subs.begin());
			set<unsigned> subs_moving;
			unsigned m = n;
			bool is_first = true;
			for (set<unsigned>::iterator it = curr_subs.begin(); it != curr_subs.end(); ++it) {
				if (is_first)
					 is_first = false;
				else {
					if (m % 2)
						subs_moving += *it;
					else
						subs_staying += *it;
					m /= 2;
				}
			}
			curr_partition[pr.first] = subs_staying;
			curr_partition[source_partition.rbegin()->first + 1] = subs_moving;
			ret += curr_partition;
		}
	}

	return ret;
}

}   // namespace partition 

